@@ -5,13 +5,16 @@
 """A keyboard-driven, vim-like browser based on Python and Qt."""
 
 import os.path
+import datetime
+
+_year = datetime.date.today().year
 
 __author__ = "Florian Bruhin"
-__copyright__ = "Copyright 2014-2021 Florian Bruhin (The Compiler)"
+__copyright__ = "Copyright 2013-{} Florian Bruhin (The Compiler)".format(_year)
 __license__ = "GPL"
 __maintainer__ = __author__
 __email__ = "mail@qutebrowser.org"
-__version__ = "3.0.0"
+__version__ = "3.2.1"
 __version_info__ = tuple(int(part) for part in __version__.split('.'))
 __description__ = "A keyboard-driven, vim-like browser based on Python and Qt."
 
