@@ -132,6 +132,9 @@ def init(*, args: argparse.Namespace) -> None:
     crashsignal.crash_handler.init_faulthandler()
 
     objects.qapp.setQuitOnLastWindowClosed(False)
+    # WORKAROUND for KDE file dialogs / QEventLoopLocker quitting:
+    # https://bugreports.qt.io/browse/QTBUG-124386
+    objects.qapp.setQuitLockEnabled(False)
     quitter.instance.shutting_down.connect(QApplication.closeAllWindows)
 
     _init_icon()
@@ -353,6 +356,10 @@ def _open_special_pages(args):
              objects.backend != usertypes.Backend.QtWebKit
          ),
          'qute://warning/qt5'),
+
+        ('birthday-2023-shown',
+         datetime.date.today() == datetime.date(2023, 12, 14),
+         'https://qutebrowser.org/birthday.html'),
     ]
 
     if 'quickstart-done' not in general_sect:
