@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """A HintManager to draw hints over links."""
 
@@ -30,15 +15,15 @@ from string import ascii_lowercase
 from typing import (TYPE_CHECKING, Callable, Dict, Iterable, Iterator, List, Mapping,
                     MutableSequence, Optional, Sequence, Set)
 
-from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject, Qt, QUrl
-from PyQt5.QtWidgets import QLabel
+from qutebrowser.qt.core import pyqtSignal, pyqtSlot, QObject, Qt, QUrl
+from qutebrowser.qt.widgets import QLabel
 
 from qutebrowser.config import config, configexc
 from qutebrowser.keyinput import modeman, modeparsers, basekeyparser
 from qutebrowser.browser import webelem, history
 from qutebrowser.commands import runners
 from qutebrowser.api import cmdutils
-from qutebrowser.utils import usertypes, log, qtutils, message, objreg, utils
+from qutebrowser.utils import usertypes, log, qtutils, message, objreg, utils, urlutils
 if TYPE_CHECKING:
     from qutebrowser.browser import browsertab
 
@@ -92,12 +77,12 @@ class HintLabel(QLabel):
         self._context = context
         self.elem = elem
 
-        self.setTextFormat(Qt.RichText)
+        self.setTextFormat(Qt.TextFormat.RichText)
 
         # Make sure we can style the background via a style sheet, and we don't
         # get any extra text indent from Qt.
         # The real stylesheet lives in mainwindow.py for performance reasons..
-        self.setAttribute(Qt.WA_StyledBackground, True)
+        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, True)
         self.setIndent(0)
 
         self._context.tab.contents_size_changed.connect(self._move_to_elem)
@@ -252,10 +237,10 @@ class HintActions:
         sel = (context.target == Target.yank_primary and
                utils.supports_selection())
 
-        flags = QUrl.FullyEncoded | QUrl.RemovePassword
+        flags = urlutils.FormatOption.ENCODED | urlutils.FormatOption.REMOVE_PASSWORD
         if url.scheme() == 'mailto':
-            flags |= QUrl.RemoveScheme
-        urlstr = url.toString(flags)  # type: ignore[arg-type]
+            flags |= urlutils.FormatOption.REMOVE_SCHEME
+        urlstr = url.toString(flags)
 
         new_content = urlstr
 
@@ -276,22 +261,21 @@ class HintActions:
 
     def run_cmd(self, url: QUrl, context: HintContext) -> None:
         """Run the command based on a hint URL."""
-        urlstr = url.toString(QUrl.FullyEncoded)  # type: ignore[arg-type]
+        urlstr = url.toString(urlutils.FormatOption.ENCODED)
         args = context.get_args(urlstr)
         commandrunner = runners.CommandRunner(self._win_id)
         commandrunner.run_safely(' '.join(args))
 
     def preset_cmd_text(self, url: QUrl, context: HintContext) -> None:
         """Preset a commandline text based on a hint URL."""
-        flags = QUrl.FullyEncoded
-        urlstr = url.toDisplayString(flags)  # type: ignore[arg-type]
+        urlstr = url.toDisplayString(urlutils.FormatOption.ENCODED)
         args = context.get_args(urlstr)
         text = ' '.join(args)
         if text[0] not in modeparsers.STARTCHARS:
             raise HintingError("Invalid command text '{}'.".format(text))
 
         cmd = objreg.get('status-command', scope='window', window=self._win_id)
-        cmd.set_cmd_text(text)
+        cmd.cmd_set_text(text)
 
     def download(self, elem: webelem.AbstractWebElement,
                  context: HintContext) -> None:
@@ -325,19 +309,18 @@ class HintActions:
 
         cmd = context.args[0]
         args = context.args[1:]
-        flags = QUrl.FullyEncoded
+        flags = urlutils.FormatOption.ENCODED
 
         env = {
             'QUTE_MODE': 'hints',
             'QUTE_SELECTED_TEXT': str(elem),
             'QUTE_SELECTED_HTML': elem.outer_xml(),
-            'QUTE_CURRENT_URL':
-                context.baseurl.toString(flags),  # type: ignore[arg-type]
+            'QUTE_CURRENT_URL': context.baseurl.toString(flags),
         }
 
         url = elem.resolve_url(context.baseurl)
         if url is not None:
-            env['QUTE_URL'] = url.toString(flags)  # type: ignore[arg-type]
+            env['QUTE_URL'] = url.toString(flags)
 
         try:
             userscripts.run_async(context.tab, cmd, *args, win_id=self._win_id,
@@ -357,7 +340,7 @@ class HintActions:
             context: The HintContext to use.
         """
         urlstr = url.toString(
-            QUrl.FullyEncoded | QUrl.RemovePassword)  # type: ignore[arg-type]
+            QUrl.ComponentFormattingOption.FullyEncoded | QUrl.UrlFormattingOption.RemovePassword)
         args = context.get_args(urlstr)
         commandrunner = runners.CommandRunner(self._win_id)
         commandrunner.run_safely('spawn ' + ' '.join(args))
@@ -657,6 +640,7 @@ class HintManager(QObject):
             self._context.labels[string] = label
 
         keyparser = self._get_keyparser(usertypes.KeyMode.hint)
+        assert isinstance(keyparser, modeparsers.HintKeyParser), keyparser
         keyparser.update_bindings(strings)
 
         modeman.enter(self._win_id, usertypes.KeyMode.hint,
@@ -852,6 +836,7 @@ class HintManager(QObject):
             # apply auto_follow_timeout
             timeout = config.val.hints.auto_follow_timeout
             normal_parser = self._get_keyparser(usertypes.KeyMode.normal)
+            assert isinstance(normal_parser, modeparsers.NormalKeyParser), normal_parser
             normal_parser.set_inhibited_timeout(timeout)
             # unpacking gets us the first (and only) key in the dict.
             self._fire(*visible)
@@ -927,6 +912,7 @@ class HintManager(QObject):
                 self._context.labels[string] = label
 
             keyparser = self._get_keyparser(usertypes.KeyMode.hint)
+            assert isinstance(keyparser, modeparsers.HintKeyParser), keyparser
             keyparser.update_bindings(strings, preserve_filter=True)
 
             # Note: filter_hints can be called with non-None filterstr only
