@@ -1,36 +1,21 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Our own QNetworkAccessManager."""
 
 import collections
 import html
 import dataclasses
-from typing import TYPE_CHECKING, Dict, MutableMapping, Optional, Sequence
+from typing import TYPE_CHECKING, Dict, MutableMapping, Optional, Set
 
-from PyQt5.QtCore import pyqtSlot, pyqtSignal, QUrl, QByteArray
-from PyQt5.QtNetwork import (QNetworkAccessManager, QNetworkReply, QSslSocket,
-                             QSslError, QNetworkProxy)
+from qutebrowser.qt.core import pyqtSlot, pyqtSignal, QUrl, QByteArray
+from qutebrowser.qt.network import (QNetworkAccessManager, QNetworkReply, QSslConfiguration,
+                             QNetworkProxy)
 
 from qutebrowser.config import config
 from qutebrowser.utils import (message, log, usertypes, utils, objreg,
-                               urlutils, debug)
+                               urlutils, debug, qtlog)
 from qutebrowser.browser import shared
 from qutebrowser.browser.network import proxy as proxymod
 from qutebrowser.extensions import interceptors
@@ -79,7 +64,7 @@ def _is_secure_cipher(cipher):
         return False
     # OpenSSL should already protect against this in a better way
     # elif (('CBC3' in tokens or 'CBC' in tokens) and (cipher.protocol() not in
-    #         [QSsl.TlsV1_0, QSsl.TlsV1_1, QSsl.TlsV1_2])):
+    #         [QSsl.SslProtocol.TlsV1_0, QSsl.SslProtocol.TlsV1_1, QSsl.SslProtocol.TlsV1_2])):
     #     # https://en.wikipedia.org/wiki/POODLE
     #     return False
     ### These things should never happen as those are already filtered out by
@@ -103,7 +88,8 @@ def _is_secure_cipher(cipher):
 
 def init():
     """Disable insecure SSL ciphers on old Qt versions."""
-    default_ciphers = QSslSocket.defaultCiphers()
+    ssl_config = QSslConfiguration.defaultConfiguration()
+    default_ciphers = ssl_config.ciphers()
     log.init.vdebug(  # type: ignore[attr-defined]
         "Default Qt ciphers: {}".format(
             ', '.join(c.name() for c in default_ciphers)))
@@ -119,10 +105,13 @@ def init():
     if bad_ciphers:
         log.init.debug("Disabling bad ciphers: {}".format(
             ', '.join(c.name() for c in bad_ciphers)))
-        QSslSocket.setDefaultCiphers(good_ciphers)
+        ssl_config.setCiphers(good_ciphers)
 
 
-_SavedErrorsType = MutableMapping[urlutils.HostTupleType, Sequence[QSslError]]
+_SavedErrorsType = MutableMapping[
+    urlutils.HostTupleType,
+    Set[certificateerror.CertificateErrorWrapper],
+]
 
 
 class NetworkManager(QNetworkAccessManager):
@@ -141,8 +130,8 @@ class NetworkManager(QNetworkAccessManager):
                  (or None for generic network managers)
         _tab_id: The tab ID this NetworkManager is associated with.
                  (or None for generic network managers)
-        _rejected_ssl_errors: A {QUrl: [SslError]} dict of rejected errors.
-        _accepted_ssl_errors: A {QUrl: [SslError]} dict of accepted errors.
+        _rejected_ssl_errors: A {QUrl: {SslError}} dict of rejected errors.
+        _accepted_ssl_errors: A {QUrl: {SslError}} dict of accepted errors.
         _private: Whether we're in private browsing mode.
         netrc_used: Whether netrc authentication was performed.
 
@@ -154,7 +143,7 @@ class NetworkManager(QNetworkAccessManager):
 
     def __init__(self, *, win_id, tab_id, private, parent=None):
         log.init.debug("Initializing NetworkManager")
-        with log.disable_qt_msghandler():
+        with qtlog.disable_qt_msghandler():
             # WORKAROUND for a hang when a message is printed - See:
             # https://www.riverbankcomputing.com/pipermail/pyqt/2014-November/035045.html
             #
@@ -172,8 +161,8 @@ class NetworkManager(QNetworkAccessManager):
         self._set_cookiejar()
         self._set_cache()
         self.sslErrors.connect(self.on_ssl_errors)
-        self._rejected_ssl_errors: _SavedErrorsType = collections.defaultdict(list)
-        self._accepted_ssl_errors: _SavedErrorsType = collections.defaultdict(list)
+        self._rejected_ssl_errors: _SavedErrorsType = collections.defaultdict(set)
+        self._accepted_ssl_errors: _SavedErrorsType = collections.defaultdict(set)
         self.authenticationRequired.connect(self.on_authentication_required)
         self.proxyAuthenticationRequired.connect(self.on_proxy_authentication_required)
         self.netrc_used = False
@@ -214,25 +203,47 @@ class NetworkManager(QNetworkAccessManager):
             abort_on.append(tab.load_started)
         return abort_on
 
+    def _get_tab(self):
+        """Get the tab this NAM is associated with.
+
+        Return:
+            The tab if available, None otherwise.
+        """
+        # There are some scenarios where we can't figure out current_url:
+        # - There's a generic NetworkManager, e.g. for downloads
+        # - The download was in a tab which is now closed.
+        if self._tab_id is None:
+            return None
+
+        assert self._win_id is not None
+        try:
+            return objreg.get('tab', scope='tab', window=self._win_id, tab=self._tab_id)
+        except KeyError:
+            # https://github.com/qutebrowser/qutebrowser/issues/889
+            return None
+
     def shutdown(self):
         """Abort all running requests."""
-        self.setNetworkAccessible(QNetworkAccessManager.NotAccessible)
+        try:
+            self.setNetworkAccessible(QNetworkAccessManager.NetworkAccessibility.NotAccessible)
+        except AttributeError:
+            # Qt 5 only, deprecated seemingly without replacement.
+            pass
         self.shutting_down.emit()
 
     # No @pyqtSlot here, see
     # https://github.com/qutebrowser/qutebrowser/issues/2213
-    def on_ssl_errors(self, reply, errors):  # noqa: C901 pragma: no mccabe
+    def on_ssl_errors(self, reply, qt_errors):
         """Decide if SSL errors should be ignored or not.
 
         This slot is called on SSL/TLS errors by the self.sslErrors signal.
 
         Args:
             reply: The QNetworkReply that is encountering the errors.
-            errors: A list of errors.
+            qt_errors: A list of errors.
         """
-        errors = [certificateerror.CertificateErrorWrapper(e) for e in errors]
-        log.network.debug("Certificate errors: {!r}".format(
-            ' / '.join(str(err) for err in errors)))
+        errors = certificateerror.CertificateErrorWrapper(reply, qt_errors)
+        log.network.debug("Certificate errors: {!r}".format(errors))
         try:
             host_tpl: Optional[urlutils.HostTupleType] = urlutils.host_tuple(
                 reply.url())
@@ -242,10 +253,8 @@ class NetworkManager(QNetworkAccessManager):
             is_rejected = False
         else:
             assert host_tpl is not None
-            is_accepted = set(errors).issubset(
-                self._accepted_ssl_errors[host_tpl])
-            is_rejected = set(errors).issubset(
-                self._rejected_ssl_errors[host_tpl])
+            is_accepted = errors in self._accepted_ssl_errors[host_tpl]
+            is_rejected = errors in self._rejected_ssl_errors[host_tpl]
 
         log.network.debug("Already accepted: {} / "
                           "rejected {}".format(is_accepted, is_rejected))
@@ -257,15 +266,22 @@ class NetworkManager(QNetworkAccessManager):
             return
 
         abort_on = self._get_abort_signals(reply)
-        ignore = shared.ignore_certificate_errors(reply.url(), errors,
-                                                  abort_on=abort_on)
-        if ignore:
-            reply.ignoreSslErrors()
-            err_dict = self._accepted_ssl_errors
-        else:
-            err_dict = self._rejected_ssl_errors
-        if host_tpl is not None:
-            err_dict[host_tpl] += errors
+
+        tab = self._get_tab()
+        first_party_url = QUrl() if tab is None else tab.data.last_navigation.url
+
+        shared.handle_certificate_error(
+            request_url=reply.url(),
+            first_party_url=first_party_url,
+            error=errors,
+            abort_on=abort_on,
+        )
+
+        if errors.certificate_was_accepted():
+            if host_tpl is not None:
+                self._accepted_ssl_errors[host_tpl].add(errors)
+        elif host_tpl is not None:
+            self._rejected_ssl_errors[host_tpl].add(errors)
 
     def clear_all_ssl_errors(self):
         """Clear all remembered SSL errors."""
@@ -353,10 +369,10 @@ class NetworkManager(QNetworkAccessManager):
             if referer_header_conf == 'never':
                 # Note: using ''.encode('ascii') sends a header with no value,
                 # instead of no header at all
-                req.setRawHeader('Referer'.encode('ascii'), QByteArray())
+                req.setRawHeader(b'Referer', QByteArray())
             elif (referer_header_conf == 'same-domain' and
                   not urlutils.same_domain(req.url(), current_url)):
-                req.setRawHeader('Referer'.encode('ascii'), QByteArray())
+                req.setRawHeader(b'Referer', QByteArray())
             # If refer_header_conf is set to 'always', we leave the header
             # alone as QtWebKit did set it.
         except urlutils.InvalidUrlError:
@@ -379,42 +395,34 @@ class NetworkManager(QNetworkAccessManager):
             proxy_error = proxymod.application_factory.get_error()
             if proxy_error is not None:
                 return networkreply.ErrorNetworkReply(
-                    req, proxy_error, QNetworkReply.UnknownProxyError,
+                    req, proxy_error, QNetworkReply.NetworkError.UnknownProxyError,
                     self)
 
         if not req.url().isValid():
             log.network.debug("Ignoring invalid requested URL: {}".format(
                 req.url().errorString()))
             return networkreply.ErrorNetworkReply(
-                req, "Invalid request URL", QNetworkReply.HostNotFoundError,
+                req, "Invalid request URL", QNetworkReply.NetworkError.HostNotFoundError,
                 self)
 
         for header, value in shared.custom_headers(url=req.url()):
             req.setRawHeader(header, value)
 
-        # There are some scenarios where we can't figure out current_url:
-        # - There's a generic NetworkManager, e.g. for downloads
-        # - The download was in a tab which is now closed.
+        tab = self._get_tab()
         current_url = QUrl()
-
-        if self._tab_id is not None:
-            assert self._win_id is not None
+        if tab is not None:
             try:
-                tab = objreg.get('tab', scope='tab', window=self._win_id,
-                                 tab=self._tab_id)
                 current_url = tab.url()
-            except (KeyError, RuntimeError):
-                # https://github.com/qutebrowser/qutebrowser/issues/889
-                # Catching RuntimeError because we could be in the middle of
-                # the webpage shutdown here.
-                current_url = QUrl()
+            except RuntimeError:
+                # We could be in the middle of the webpage shutdown here.
+                pass
 
         request = interceptors.Request(first_party_url=current_url,
                                        request_url=req.url())
         interceptors.run(request)
         if request.is_blocked:
             return networkreply.ErrorNetworkReply(
-                req, HOSTBLOCK_ERROR_STRING, QNetworkReply.ContentAccessDenied,
+                req, HOSTBLOCK_ERROR_STRING, QNetworkReply.NetworkError.ContentAccessDenied,
                 self)
 
         if 'log-requests' in objects.debug_flags:
