@@ -414,7 +414,7 @@ class CompletionView(QTreeView):
     def on_clear_completion_selection(self):
         """Clear the selection model when an item is activated."""
         self.hide()
-        selmod = self._selection_model()
+        selmod = self.selectionModel()
         if selmod is not None:
             selmod.clearSelection()
             selmod.clearCurrentIndex()
@@ -437,8 +437,7 @@ class CompletionView(QTreeView):
             contents_height = (
                 self.viewportSizeHint().height() +
                 bar.sizeHint().height())
-            if contents_height <= height:
-                height = contents_height
+            height = min(height, contents_height)
         # The width isn't really relevant as we're expanding anyways.
         return QSize(-1, height)
 
