@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Functions related to host blocking."""
 
@@ -26,7 +11,7 @@ import logging
 import pathlib
 from typing import cast, IO, Set
 
-from PyQt5.QtCore import QUrl
+from qutebrowser.qt.core import QUrl
 
 from qutebrowser.api import (
     hook,
@@ -64,9 +49,10 @@ def get_fileobj(byte_io: IO[bytes]) -> IO[bytes]:
     byte_io.seek(0)  # rewind downloaded file
     if zipfile.is_zipfile(byte_io):
         byte_io.seek(0)  # rewind what zipfile.is_zipfile did
-        zf = zipfile.ZipFile(byte_io)
-        filename = _guess_zip_filename(zf)
-        byte_io = zf.open(filename, mode="r")
+        with zipfile.ZipFile(byte_io) as zf:
+            filename = _guess_zip_filename(zf)
+            # pylint: disable=consider-using-with
+            byte_io = zf.open(filename, mode="r")
     else:
         byte_io.seek(0)  # rewind what zipfile.is_zipfile did
     return byte_io
