@@ -483,6 +483,8 @@ class MainWindow(QWidget):
         mode_manager = modeman.instance(self.win_id)
 
         # misc
+        self._prompt_container.release_focus.connect(
+            self.tabbed_browser.on_release_focus)
         self.tabbed_browser.close_window.connect(self.close)
         mode_manager.entered.connect(hints.on_mode_entered)
 
@@ -559,6 +561,7 @@ class MainWindow(QWidget):
             self._completion.on_clear_completion_selection)
         self.status.cmd.hide_completion.connect(
             self._completion.hide)
+        self.status.cmd.hide_cmd.connect(self.tabbed_browser.on_release_focus)
 
     def _set_decoration(self, hidden):
         """Set the visibility of the window decoration via Qt."""
