@@ -398,7 +398,7 @@ class TabBar(QTabBar):
         self.setStyle(self._our_style)
         self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         self.vertical = False
-        self._auto_hide_timer = QTimer()
+        self._auto_hide_timer = usertypes.Timer()
         self._auto_hide_timer.setSingleShot(True)
         self._auto_hide_timer.timeout.connect(self.maybe_hide)
         self._on_show_switching_delay_changed()
@@ -801,6 +801,11 @@ class TabBarStyle(QProxyStyle):
 
     ICON_PADDING = 4
 
+    def __init__(self, style=None):
+        # "useless" override as WORKAROUND for
+        # https://www.riverbankcomputing.com/pipermail/pyqt/2023-September/045510.html
+        super().__init__(style)
+
     def _base_style(self) -> QStyle:
         """Get the base style."""
         style = self.baseStyle()
