@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Resources related utilities."""
 
@@ -24,28 +9,34 @@ import sys
 import contextlib
 import posixpath
 import pathlib
-from typing import Iterator, Iterable
+from typing import Iterator, Iterable, Union, Dict
 
 
-# We cannot use the stdlib version on 3.7-3.8 because we need the files() API.
-if sys.version_info >= (3, 9):
+# We cannot use the stdlib version on 3.8 because we need the files() API.
+if sys.version_info >= (3, 11):  # pragma: no cover
+    # https://github.com/python/cpython/issues/90276
     import importlib.resources as importlib_resources
+    from importlib.resources.abc import Traversable
+elif sys.version_info >= (3, 9):
+    import importlib.resources as importlib_resources
+    from importlib.abc import Traversable
 else:  # pragma: no cover
     import importlib_resources
+    from importlib_resources.abc import Traversable
 
 import qutebrowser
-_cache = {}
+_cache: Dict[str, str] = {}
+_bin_cache: Dict[str, bytes] = {}
 
-def _path(filename: str) -> pathlib.Path:
+
+_ResourceType = Union[Traversable, pathlib.Path]
+
+
+def _path(filename: str) -> _ResourceType:
     """Get a pathlib.Path object for a resource."""
     assert not posixpath.isabs(filename), filename
     assert os.path.pardir not in filename.split(posixpath.sep), filename
 
-    if hasattr(sys, 'frozen'):
-        # For PyInstaller, where we can't store resource files in a qutebrowser/ folder
-        # because the executable is already named "qutebrowser" (at least on macOS).
-        return pathlib.Path(sys.executable).parent / filename
-
     return importlib_resources.files(qutebrowser) / filename
 
 @contextlib.contextmanager
@@ -64,7 +55,7 @@ def _keyerror_workaround() -> Iterator[None]:
 
 
 def _glob(
-    resource_path: pathlib.Path,
+    resource_path: _ResourceType,
     subdir: str,
     ext: str,
 ) -> Iterable[str]:
@@ -77,14 +68,11 @@ def _glob(
     glob_path = resource_path / subdir
 
     if isinstance(resource_path, pathlib.Path):
+        assert isinstance(glob_path, pathlib.Path)
         for full_path in glob_path.glob(f'*{ext}'):  # . is contained in ext
             yield full_path.relative_to(resource_path).as_posix()
-    else:  # zipfile.Path or importlib_resources compat object
-        # Unfortunately, we can't tell mypy about resource_path being of type
-        # Union[pathlib.Path, zipfile.Path] because we set "python_version = 3.6" in
-        # .mypy.ini, but the zipfile stubs (correctly) only declare zipfile.Path with
-        # Python 3.8...
-        assert glob_path.is_dir(), glob_path  # type: ignore[unreachable]
+    else:  # zipfile.Path or other importlib_resources.abc.Traversable
+        assert glob_path.is_dir(), glob_path
         for subpath in glob_path.iterdir():
             if subpath.name.endswith(ext):
                 yield posixpath.join(subdir, subpath.name)
@@ -101,6 +89,10 @@ def preload() -> None:
         for name in _glob(resource_path, subdir, ext):
             _cache[name] = read_file(name)
 
+    for name in _glob(resource_path, 'img', '.png'):
+        # e.g. broken_qutebrowser_logo.png
+        _bin_cache[name] = read_file_binary(name)
+
 
 def read_file(filename: str) -> str:
     """Get the contents of a file contained with qutebrowser.
@@ -128,6 +120,9 @@ def read_file_binary(filename: str) -> bytes:
     Return:
         The file contents as a bytes object.
     """
+    if filename in _bin_cache:
+        return _bin_cache[filename]
+
     path = _path(filename)
     with _keyerror_workaround():
         return path.read_bytes()
